//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef NEO_EXCEPTION_H
#define NEO_EXCEPTION_H

#include <string>

/**
 * @memo <DL><DD>Super class from which all NeoExceptions inherit.</DD></DL>
 */
class NeoException
{
public:
    char msg_m[2048];

	/**
	 * @memo <DL><DD>Gets the message associated with this exception.</DD></DL>
	 * @doc <br/>Gets the message associated with this exception.
	 * 
	 * @return The exception message.
	 */
    inline const char* getMessage()
    {
        return msg_m;
    }


    inline NeoException(const char* msg)
    {
        strncpy(msg_m, msg, sizeof(msg_m));
		msg_m[sizeof(msg_m)-1] = '\0';  //--just to be sure--
    }
	
	inline NeoException(const char* inUnformattedMsg, const char* inErrorName, int inErrorNumber, bool inXMLFormatted)
    {
		if(inXMLFormatted==true)
		{
			char* cpFormattedMsg = NeoException::XMLFormatted(inUnformattedMsg, inErrorName, inErrorNumber);

			strncpy(msg_m, cpFormattedMsg, sizeof(msg_m));
			msg_m[sizeof(msg_m) - 1] = '\0';  //--just to be sure--
			
			//--Free the memory--
			delete cpFormattedMsg;
		}
		else
		{
			strncpy(msg_m, inUnformattedMsg, sizeof(msg_m));
			msg_m[sizeof(msg_m) - 1] = '\0';  //--just to be sure--
		}
    }

	static inline char* XMLFormatted(const char* inUnformattedMsg, const char* inErrorName, int inErrorNumber)
	{
		char* cpFormattedMsg = new char[1024];
		memset(cpFormattedMsg, 0, 1024);

		sprintf(cpFormattedMsg,
			"<Error>\n\t<Name>%s</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", 
			inErrorName, 
			inUnformattedMsg, 
			inErrorNumber);
		return cpFormattedMsg;

	}
	
};


/**
 * @memo <DL><DD>This is thrown when the server cannot authorize against the given username and password.</DD></DL>
 * @doc <br/> This is thrown when the server cannot authorize against the given username and password.
 */
class NeoAuthorizationFailedException : public NeoException
{
public:
	inline NeoAuthorizationFailedException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Authorization Failed", 2, inXMLFormatted) {}

};

/**
 * @memo <DL><DD>This will be thrown when a socket error occurs.</DD></DL>
 * @doc <br/>This will be thrown when a socket error occurs.  Either a connection to the
 * server is lost, or a read/write error happens.
 */
class NeoCommunicationException : public NeoException
{
public:
	NeoCommunicationException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Communication Error", 3, inXMLFormatted) {}
	
};

/**
 * @memo <DL><DD>This is thrown if the NeoConnectionPool is used in an incorrect manner.</DD></DL>
 * @doc <br/>This is thrown if the NeoConnectionPool is used in an incorrect manner.
 * 
 * @author  Sandor Dornbush
 */
class NeoConnectionPoolException : public NeoException
{
public:
	NeoConnectionPoolException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Connection Pool Error", 0, inXMLFormatted) {}
};

/**
 * @memo <DL><DD>This is thrown if a user tries to call a method with an arguement value that 
 * is not valid.</DD></DL>
 * @doc <br/>This is thrown if a user tries to call a method with an arguement value that 
 * is not valid.  For example: If a parameter is blank or null and
 * the method requires a valid value, this exception will be thrown.
 */
class NeoInvalidArgumentException : public NeoException
{
public:
	NeoInvalidArgumentException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Invalid Argument", 1, inXMLFormatted) {}
		
		
		
		//) : NeoException(msg) {}
};

/**
 * @memo <DL><DD>This is thrown when the session ID has expired or does not exist.</DD></DL>
 * @doc <br/>This is thrown when the session ID has expired or does not exist.
 */
class NeoInvalidSessionException : public NeoException
{
public:
	NeoInvalidSessionException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Invalid Session ID", 4, inXMLFormatted) {}
};

/**
 * @memo <DL><DD>This is thrown when a malformed XPath or XQuery statement is passed in.</DD></DL>
 * @doc <br/>This is thrown when a malformed XPath or XQuery statement is passed in.
 */
class NeoMalformedXPathException : public NeoException
{
public:
	NeoMalformedXPathException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Malformed Xpath", 7, inXMLFormatted) {}
};

/**
 * @memo <DL><DD>This is used if a user passes in malformed XML to the XMS server.</DD></DL>
 * @doc <br/>This is used if a user passes in malformed XML to the XMS server.
 */
class NeoMalformedXMLException : public NeoException
{
public:
	NeoMalformedXMLException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Malformed XML", 6, inXMLFormatted) {}
} ;

 
/**
 * @memo <DL><DD>This will be thrown when a malformed URL is generated.</DD></DL>
 * @doc <br/>This will be thrown when a malformed URL is generated.
 * 
 */
class NeoMalformedURLException : public NeoException
{
public:
	NeoMalformedURLException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Malformed URL", 0, inXMLFormatted) {}
} ;

/**
 * @memo <DL><DD>This will be thrown on the Store commands when a specified file cannot be found.</DD></DL>
 * @doc <br/>This will be thrown on the Store commands when a specified file cannot be found.
 * 
 */
class NeoNoFileFoundException : public NeoException
{
public:
	NeoNoFileFoundException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "File Not Found", 0, inXMLFormatted) {}
} ;

/**
 * @memo <DL><DD>This is thrown if a program asks a pool for a connection after the closeAllConnections method has been called.</DD></DL>
 * @doc <br/>This is thrown if a program asks a pool for a connection after the closeAllConnections method has been called.
 */
class NeoPoolClosedException : public NeoConnectionPoolException
{
public:
	NeoPoolClosedException (const char* msg, bool inXMLFormatted = false) : 
		NeoConnectionPoolException(msg, inXMLFormatted) {}
};

/**
 * @memo <DL><DD>This is thrown when there is an error on the server processing your request.</DD></DL>
 * @doc <br/>This is thrown when there is an error on the server processing your request.
 */
class NeoServerException : public NeoException
{
public:
	NeoServerException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "NeoServerException", 8, inXMLFormatted) {}
};

/**
 * @memo <DL><DD>This is thrown if the server returns any error in relation to a transaction.</DD></DL>
 * @doc <br/>This is thrown if the server returns any error in relation to a transaction.
 */
class NeoTransactionException : public NeoException
{
public:
	NeoTransactionException (const char* msg) : NeoException(msg) {}
};
 
/**
 * @memo <DL><DD>This is thrown if a the server hits a deadlock servicing a request.</DD></DL>
 * @doc <br/>This is thrown if a the server hits a deadlock servicing a request.
 */
class NeoTransactionDeadlockException : public NeoTransactionException
{
public:
	NeoTransactionDeadlockException (const char* msg) : NeoTransactionException(msg) {}
};

/**
 * @memo <DL><DD>This is thrown if a transaction roll-back occurs on the server.</DD></DL>
 * @doc <br/>This is thrown if a transaction roll-back occurs on the server.
 */
class NeoTransactionRollbackException : public NeoTransactionException
{
public:
	NeoTransactionRollbackException (const char* msg) : NeoTransactionException(msg) {}
};
 
/**
 * @memo <DL><DD>This is thrown as a result of a timeout on a transaction.</DD></DL>
 * @doc <br/>This is thrown as a result of a timeout on a transaction.
 */
class NeoTransactionTimeoutException : public NeoTransactionException
{
public:
	NeoTransactionTimeoutException (const char* msg) : NeoTransactionException(msg) {}
};


 /** 
  * @memo <DL><DD>This is thrown when an unexpected error condition occurs in the NeoCore XMS Client. </DD></DL>
  * @doc <br/>This is thrown when an unexpected error condition occurs in the NeoCore XMS Client. 
  *
  */
class NeoClientException : public NeoException
{
public:
	NeoClientException (const char* inUnformattedMsg, bool inXMLFormatted = false) :
		NeoException(inUnformattedMsg, "Client Exception", 0, inXMLFormatted) {}
};

/** 
 * @memo <DL><DD>This is thrown when there is an error Access Control servlet processing your request. </DD></DL>
 * @doc <br/>This is thrown when there is an error Access Control servlet processing your request. 
 */
class NeoACException : public NeoServerException
{
public:
	NeoACException (const char* msg) : NeoServerException(msg) {}

};

/**
 * @memo <DL><DD>Thrown when attempting to create a user, if that group which you are 
 * attempting to assign that user to does not exist.</DD></DL>
 * @doc <br/>Thrown when attempting to create a user, if that group which you are 
 * attempting to assign that user to does not exist.
 */
class NeoACGroupDoesNotExistException : public NeoACException
{
public:
	NeoACGroupDoesNotExistException (const char* msg) : NeoACException(msg) {}
};

/**
 * @memo <DL><DD>Thrown when attempting to create a group, if that group already exists in the database.</DD></DL>
 * @doc <br/>Thrown when attempting to create a group, if that group already exists in the database.
 */
class NeoACGroupExistsException : public NeoACException
{
public:
	NeoACGroupExistsException (const char* msg) : NeoACException(msg) {}
};

/**
 * @memo <DL><DD>Thrown when attempting to create a rule, if that rule already exists in the database.</DD></DL>
 * @doc <br/>Thrown when attempting to create a rule, if that rule already exists in the database.
 */
class NeoACRuleExistsException : public NeoACException
{
public:
	NeoACRuleExistsException (const char* msg) : NeoACException(msg) {}
};

/**
 * @memo <DL><DD>Thrown when attempting to create a user, if that user already exists in the database.</DD></DL>
 * @doc <br/>Thrown when attempting to create a user, if that user already exists in the database.
 */
class NeoACUserExistsException : public NeoACException
{
public:
	NeoACUserExistsException (const char* msg) : NeoACException(msg) {}
};

/**
 * @memo <DL><DD>Thrown when attempting to delete a group, if that group still has users assigned to it.</DD></DL>
 * @doc <br/>Thrown when attempting to delete a group, if that group still has users assigned to it.
 */
class NeoACUsersExistInGroupException : public NeoACException
{
public:
	NeoACUsersExistInGroupException (const char* msg) : NeoACException(msg) {}
};

/**
 * @memo <DL><DD>Thrown when attempting to delete a group or user, if that group or user still has access control rules associated with it.</DD></DL>
 * @doc <br/>Thrown when attempting to delete a group or user, if that group or user still has access control rules associated with it.
 */
class NeoACAssociatedRulesExistException : public NeoACException
{
public:
	NeoACAssociatedRulesExistException (const char* msg) : NeoACException(msg) {}
};


 /**
  * @memo <DL><DD>This is thrown when the client destroys a NeoResultChunk and subsequently attempts to
  * move the cursor into or requests XML from a NeoResultChunk that is in the DESTROYED state.</DD></DL>
  * 
  * @doc <br/>This is thrown when the client destroys a NeoResultChunk and subsequently attempts to
  * move the cursor into or requests XML from a NeoResultChunk that is in the DESTROYED state.
  * Once NeoResultChunks are transferred from the NeoCore XMS Server to the Client they 
  * are destroyed on the Server.
  *
  * The client can call get_ResultChunkID on this exception to determine what
  * NeoResultChunk was previously destroyed.
  *
  * @see NeoResultChunk#NeoResultChunkState
  *
  * @author Richard A. Monteleone
  */

class NeoDestroyedResultChunkException : public NeoException

{
  public:
	  NeoDestroyedResultChunkException(const int resultChunkID, const char* msg) : NeoException(msg) 
	  {
		m_resultChunkID = resultChunkID;
	  }

   /**
     * @return The ID of the NeoResultChunk that is no longer available.
	 */
   const int get_ResultChunkID() 
   {
	   return m_resultChunkID;
   }

  private:

    /**
      * The ID of the NeoResultChunk.
	  */
    int m_resultChunkID;

};

 /**
  * @memo <DL><DD>This is thrown when the NeoResultProfile resultChunkTransferMode is set to Manual and the
  * client's NeoCursor is trying to enter into a NeoResultChunk that is not in the
  * LOADED state.</DD></DL>
  *
  * @doc <br/>This is thrown when the NeoResultProfile resultChunkTransferMode is set to Manual and the
  * client's NeoCursor is trying to enter into a NeoResultChunk that is not in the
  * LOADED state.  This exception is thrown to inform the client that they 
  * must load the NeoResultChunk manually prior to moving or repositioning the NeoCursor
  * into or getting XML on the NeoResultChunk.
  *  
  * The client can call get_ResultChunkID on this exception to determine what
  * NeoResultChunk needs to be loaded.
  *
  * @see NeoResultProfile
  * @see NeoResultChunk#NeoResultChunkState
  * @see NeoCursor#reposition
  * @see NeoCursor#move
  * 
  *
  * @author Richard A. Monteleone
  */

class NeoManualLoadRequiredException : public NeoException

{
  public:
	  NeoManualLoadRequiredException(const int resultChunkID, const char* msg) : NeoException(msg) 
	  {
		m_resultChunkID = resultChunkID;
	  }

   /**
     * @return The ID of the NeoResultChunk that needs to be manually loaded. 
	 */
   const int get_ResultChunkID() 
   {
	   return m_resultChunkID;
   }

  private:

    /**
      * The ID of the NeoResultChunk.
	  */
    int m_resultChunkID;

};


 /** 
  * @memo <DL><DD>This is thrown when the client attempts to move or reposition the NeoCursor outside
  * the boundary of the NeoQueryResult.</DD></DL>
  * 
  * @doc <br/>This is thrown when the client attempts to move or reposition the NeoCursor outside
  * the boundary of the NeoQueryResult.  If the specified cursor position is higher
  * than the highest possible cursor location, get_cursorBoundary() returns the highest possible 
  * cursor position.  If the specified cursor position is lower than the lowest possible cursor 
  * position (i.e. <0), get_cursorBoundary() returns the lowest possible cursor position (i.e. 0). 
  *
  * @author Richard A. Monteleone
  */

class NeoCursorOutOfRangeException : public NeoException
{
  public:

    NeoCursorOutOfRangeException(const long cursorBoundary, const char* msg): NeoException(msg) 
	{
		 m_cursorBoundary= cursorBoundary;
	}

   /**
     *  @return The boundary (highest or lowest possible cursor location) that was violated 
	 *	by the NeoCursor.
	 */
   const long get_cursorBoundary()
   {
	   return m_cursorBoundary;
   }
  private:

    // The boundary that was violated by the NeoCursor 
    long m_cursorBoundary;

};

 
 /** 
  * @memo <DL><DD>Thrown when the user has specified a maximum Axis Elements size in the NeoResultProfile 
  * that is violated and has the exceededMode set to NeoResultProfile.REJECT (0).</DD></DL>
  *
  * @doc <br/>Thrown when the user has specified a maximum Axis Elements size in the NeoResultProfile 
  * that is violated and has the exceededMode set to NeoResultProfile.REJECT (0). 
  *
  * @see NeoResultProfile
  *
  * @author Richard A. Monteleone
  */

class NeoQueryResultExceededMaxException : public NeoException
{

public:

	NeoQueryResultExceededMaxException (const char* msg) : NeoException(msg) {}

};

/**
 * @memo <DL><DD>This will be thrown if a file could not be found or accessed properly.</DD></DL>
 * @doc <br/>This will be thrown if a file could not be found or accessed properly.
 */
class NeoFileErrorException : public NeoException
{
public:
	NeoFileErrorException (const char* msg) : NeoException(msg) {}
} ;



/**
 * @memo <DL><DD>This will be thrown if the operation prohibits duplicate entries and 
 * you attempted to add or insert something that would result in a duplicate
 * entry.</DD></DL>
 * @doc <br/>This will be thrown if the operation prohibits duplicate entries and 
 * you attempted to add or insert something that would result in a duplicate
 * entry.
 */
class NeoAttemptedDuplicateException : public NeoException
{
public:
	NeoAttemptedDuplicateException (const char* msg) : NeoException(msg) {}
} ;



// The following are defined for use by the Client to ensure that our exceptions are
// formatted consistently
#define TMP_MSG  \
	  char tmp[1024]; \
	  tmp[0] = 0x0;

#define THROW_NEOINVALID_ARGUMENT(message) \
	   TMP_MSG  \
	   sprintf(tmp,"<Error>\n\t<Name>Invalid Argument Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
			NULL != message ? message : "", 1); \
	   throw NeoInvalidArgumentException(tmp);

#define THROW_NEOCOMMUNICATION(message) \
		TMP_MSG \
		sprintf(tmp,"<Error>\n\t<Name>Communication Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
				NULL != message ? message : "", 3); \
		throw NeoCommunicationException(tmp);

#define THROW_NEOSERVER(message) \
		TMP_MSG \
		sprintf(tmp,"<Error>\n\t<Name>NeoServer Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
				NULL != message ? message : "", 8); \
		throw NeoServerException(tmp);

#define THROW_NEODESTROYED_RESULT_CHUNK(resultChunkID, message) \
		TMP_MSG \
		sprintf(tmp,"<Error>\n\t<Name>Destroyed NeoResultChunk Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
				NULL != message ? message : "", 14); \
		throw NeoDestroyedResultChunkException(resultChunkID, tmp);


#define THROW_NEOCURSOR_OUT_OF_RANGE(cursorBoundary, message) \
		TMP_MSG	\
		sprintf(tmp,"<Error>\n\t<Name>NeoCursor Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
				NULL != message ? message : "", 63); \
		throw NeoCursorOutOfRangeException(cursorBoundary, tmp);


#define THROW_MANUAL_LOAD_REQUIRED(resultChunkID, message) \
		TMP_MSG \
		sprintf(tmp,"<Error>\n\t<Name>Manual Load NeoResultChunk Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
				NULL != message ? message : "", 64); \
		throw NeoManualLoadRequiredException(resultChunkID, tmp);


#define THROW_NEOCLIENT(message) \
		TMP_MSG \
		sprintf(tmp,"<Error>\n\t<Name>NeoClient Error</Name>\n\t<Message>%s</Message>\n\t<Exception-Number>%d</Exception-Number>\n</Error>", \
				NULL != message ? message : "", 65); \
		throw NeoClientException(tmp);




#endif // NEO_EXCEPTION_H
