#ifndef RESULTCHUNK_H_INCLUDED_C3B72EEB
#define RESULTCHUNK_H_INCLUDED_C3B72EEB


#include "NeoException.h"
#include "UrlConnection.h"
#include "XMLUtils.h"
#include <vector>

using namespace std;


#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif


/**
  * @memo <DL><DD>The client-side object that holds each chunk of XML identified by a query.</DD></DL>
  * @doc <br/>The client-side object that holds each chunk of XML identified by a query. Initially the
  * NeoResultChunk's just hold metadata, as XML is transferred from the server
  * this object contains the XML. As XML is transferred from the NeoCore XMS Server to 
  * the client the chunks are deleted on the server. 
  *
  * @author Richard A. Monteleone
  *
  * @version 1.0
  *
  * @see NeoQueryResult
  * @see NeoCursor
  * @see NeoResultProfile
  *
  */  





class  HTTPCLIENT4_API NeoResultChunk
{
  public:

    /**
	  * @memo <DL><DD>NeoResultChunkState</DD></DL>
      * @doc <br/>
	  *  <UL><LI><B>-1 = (DESTROYED)</B> Previously Loaded and deleted (XML for this chunk of 
      *       Axis Elements no longer exists on the client or server).</LI>
      *   <LI><B>0 = (NEVER_LOADED)</B> Never Loaded with XML from the Server (XML exists 
      *       on the server but has never been transferred to the client. </LI>
      *   <LI><B>1 = (LOADED)</B> Loaded with XML from the Server (XML for this NeoResultChunk no
      *  	  longer exists on the server).</LI></UL>
	  */
	 enum  NeoResultChunkState {DESTROYED = -1, NEVER_LOADED = 0 , LOADED = 1}; 



    /**
	  * @memo <DL><DD>Construct a NeoResultChunk without Axis Elements.</DD></DL>
      * @doc <br/>Construct a NeoResultChunk without Axis Elements.  This constructor is used
      * when we need a NeoResultChunk but don't want to load the NeoResultChunk with
	  * axis elements. We create the NeoResultChunk with its metadata only at this point.
	  *
      * 
	  * @param sid The Session ID that was used when the initial query was executed. 
	  * @param urlConnection An established connection to the NeoCore XMS Server
	  * @param NeoQueryResultID The ID of the parent NeoQueryResult, used when communicating
	  * with the server to load and destroy NeoResultChunks
	  * @param NeoResultChunkID The ID that we will assign to this NeoResultChunk
	  * @param offset the offset that this NeoResultChunk will have in the NeoQueryResult
	  * @param numAxisElements the number of AxisElements that this NeoResultChunk can
	  * hold when fully loaded.
	  * @param autoLoadEnabled Tells us whether or not we should auto-load this NeoResultChunk
	  * if its' state is NEVER_LOADED
	  * 
	  * @exception NeoInvalidArgumentException - If any of the parameters are invalid
	  */
    NeoResultChunk(const char* const  sid, UrlConnection* urlConnection, 
				   const int NeoQueryResultID, const int NeoResultChunkID, 
				   const int offset, const int numAxisElements,
				   const bool autoLoadEnabled) 	
	throw (NeoInvalidArgumentException);



    /** 
	  * @memo <DL><DD>Loads this NeoResultChunk with the ChunkXML's Axis Elements, expected XML
	  * is in the following form, assumes well-formed XML. </DD></DL>
	  * @doc <br/>Loads this NeoResultChunk with the ChunkXML's Axis Elements, expected XML
	  * is in the following form, assumes well-formed XML. <br/><br/>
	  *
      *		&lt;ChunkXML&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;ID&gt;numeric&lt;/ID&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;AE&gt;axisElementData&lt;/AE&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;AE&gt;axisElementData&lt;/AE&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *			&lt;AE&gt;axisElementData&lt;/AE&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      *				...<br/>
      *		&lt;/ChunkXML&gt;<br/>
	  *
	  * @exception NeoInvalidArgumentException - If chunkXML is NULL or empty
      * @exception NeoClientException - Unhandled client-side error condition has occured. 
      *
	  * @return false if this NeoResultChunk is already loaded, true otherwise. 
	  */
    const bool load(const char* chunkXML)	
	throw ( NeoInvalidArgumentException,
			NeoClientException);



    /**
	  * @memo <DL><DD>Loads this NeoResultChunk.</DD></DL>
	  * @doc <br/>Loads this NeoResultChunk. If auto-load is enabled and this NeoResultChunk has 
	  * never been loaded (i.e. NEVER_LOADED state) then this method will communicate with
	  * the NeoCore XMS to load it. If auto-load is disabled and this NeoResultChunk has
	  * never been loaded we throw an exception to inform the user that they must manually load
	  * it. 
	  *	  
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed 
	  * this NeoQueryResult is no longer valid
	  * @exception NeoDestroyedResultChunkException - If this NeoResultChunk was previously loaded and destroyed  
      * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server
      * @exception NeoClientException - Unhandled client error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  * @see NeoResultChunkState
	  *
	  * @return true if the NeoResultChunk was loaded, false otherwise.
	  * (NeoResultChunk already has a state of LOADED.)
	  */	
    const bool load()      
	throw ( NeoDestroyedResultChunkException,
			NeoInvalidSessionException,
			NeoCommunicationException,
			NeoClientException, 
			NeoServerException);



    /**
	  * @memo <DL><DD>Takes the cursor position provided and determines if the cursor
      * position is located within this NeoResultChunk. </DD></DL>
	  * @doc <br/>Takes the cursor position provided and determines if the cursor
      * position is located within this NeoResultChunk. 
	  * 
	  * @param cursorPosition Absolute cursor postion
	  *
	  * @return true if this cursorPosition resides in this NeoResultChunk, false
	  * otherwise.
      */
    const bool cursorOutsideResultChunk(const int cursorPosition) const;




    /**
	  * @memo <DL><DD>Retrieves  XML contained by this NeoResultChunk.</DD></DL>
	  * @doc <br/>
      * Retrieves  XML contained by this NeoResultChunk. The XML will be wrapped in the 
      * TAG_WRAPPER_START and TAG_WRAPPER_END XML tag. If autoLoadEnabled is true and this
      * NeoResultChunk is in the state NEVER_LOADED then it will be loaded automatically
      * and the XML will be returned. 
      *
      * @exception NeoManualLoadRequiredException - If the data for this NeoResultChunk has not been loaded 
      * @exception NeoInvalidSessionException - If the session that was used for the initial query that constructed this NeoQueryResult
      * is no longer valid 
      * @exception NeoDestroyedResultChunkException - If t NeoResultChunk was previously loaded and destroyed  
      * @exception NeoCommunicationException - If there was a problem communicating with the server to load (auto-mode) the NeoResultChunk 
      * @exception NeoClientException - Unhandled client error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  * @see NeoResultProfile
	  *
	  * @return The XML contained by this NeoResultChunk, example XML:
      *
      *	</Query-Results>
	  *    <YourData>1</YourData>
	  *    <YourData>1</YourData>
	  * </Query-Results>
	  *
	  */	
    char* get_XML()  
	throw ( NeoDestroyedResultChunkException,
	   	    NeoInvalidSessionException,
		    NeoManualLoadRequiredException,
		    NeoCommunicationException,
		    NeoClientException,
			NeoServerException);





    /**
	  * Retrieves the size of this NeoResultChunk, how many axis elements it
	  * can hold when loaded. It does not consider its current state.
	  *
	  * @return number of axis elements
	  */
	const int get_axisElementSize() const;




    /**
	  * Retrieves the XML starting at relativeStaringPosition plus the number of additionalElements. 
	  * If autoLoadEnabled is true and this NeoResultChunk is in the state NEVER_LOADED then it
      * will be loaded automatically and the XML will be returned. If the additionalElements 
	  * requested ventures into another chunk then remainingElements is populated with the number 
	  * of remaining elements that must be loaded to satisfy this request.
	  * 
      * @exception NeoManualLoadRequiredException - If the data for this NeoResultChunk has not been
	  * loaded and we do not have auto-load enabled 
	  * @exception NeoDestroyedResultChunkException - If this NeoResultChunk was previously loaded and 
	  * destroyed 
	  * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server
	  * to load this NeoResultChunk 
	  * @exception NeoInvalidSessionException - If the session that was used for the initial query that 
	  * constructed this NeoResultChunk is no longer valid	
      * @exception NeoClientException - Unhandled client-side error condition has occured. 
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
      *
	  * @return the reconstructed XML NOT wrapped in the TAG_WRAPPER_START and TAG_WRAPPER_END, this
	  * was done so that you don't have to strip off the wrapper tags when getting
	  * axis elements from multiple NeoResultChunks. The caller is responsible for
	  * wrapping the XML in the wrapper tags before delivering it to the user
	  */
	char* get_XML(const int relativeStartingPosition, const int additionalElements, int& remainingElements)  	 
	throw ( NeoInvalidArgumentException,
			NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);





    /**
	  * Prepares this NeoResultChunk for XML Acquisition by looking to see if this
      * NeoResultChunk is LOADED. If its's state is NEVER_LOADED and m_autoLoadEnabled 
	  * is true it loads this NeoResultChunk. 
	  *
	  * @exception NeoManualLoadRequiredException - If m_autoLoadEnabled is false and
	  * this NeoResultChunk is in the NEVER_LOADED state
	  * @exception NeoDestroyedResultChunkException - If this NeoResultChunk has a state 
	  * of DESTROYED
      * @exception NeoClientException - Unhandled client-side error condition has occured. 
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  *
	  *
	  * @return void
	  */
	void prepareForXMLAcquisition()	
	throw ( NeoInvalidSessionException,
			NeoManualLoadRequiredException,
			NeoDestroyedResultChunkException,
			NeoCommunicationException,
			NeoClientException,
			NeoServerException);


    /**
	  * Destroys this NeoResultChunk, primary use is when the client elects to
      * manually manage the memory utilized by NeoResultChunks. If this NeoResultChunk hasn't
	  * been loaded (state:NEVER_LOADED) then this method contacts the NeoXMS server to destroy
	  * the NeoResultChunk on the NeoCore XMS Server. Once a NeoResultChunk is destoryed on the
	  * server it is destroyed forever. If this NeoResultChunk is loaded (state:LOADED), then
	  * the AxisElements for it contains are detroyed (memory freed). 
	  *  . 
	  * @exception NeoCommunicationException - If there was a problem communicating with the NeoCore XMS Server
	  * to load this NeoResultChunk 
	  * @exception NeoInvalidSessionException - If the session that was used for the initial query that 
	  * constructed this NeoResultChunk is no longer valid
      * @exception NeoClientException - Unhandled client-side error condition has occured
      * @exception NeoServerException - If there was an internal NeoCore XMS error.
	  * 
	  * @return true if the NeoResultChunk was LOADED or NEVER_LOADED, false otherwise (NeoResultChunk has a state of DESTROYED)
	  */
	bool destroy() 
	throw ( NeoCommunicationException,
		    NeoInvalidSessionException,
		    NeoClientException,
			NeoServerException);	




	/** 
	  * Retrieves the current state of this NeoResultChunk
	  *	
	  * @see NeoResultChunkState
	  *
	  * @return the state of this NeoResultChunk
	  *
	  */	   
    const NeoResultChunkState get_state() const;




	/** 
	  * Retrieves the ID of this NeoResultChunk
	  *
	  * @return the ID of this NeoResultChunk
	  *
	  */
    const int get_ID() const;


	/** 
	  * Retrieves the element offset of this NeoResultChunk
	  *
	  * @return the element offset of this NeoResultChunk
	  *
	  */
    const int get_elementOffset() const;



	/** 
	  * Destructor for this NeoResultChunk.  If the state of this NeoResultChunk is LOADED
	  * the axis elements are destroyed. It is not the responsibility of this destructor
	  * to communicate with the NeoXMS Server to destroy any server-side chunks that have 
	  * not been transferred to the client.  The NeoQueryResult's destructor is responsible 
	  * for destroying all server-side result chunks.  
	  * 
	  * @see NeoQueryResult#~NeoQueryResult
      *
	  */
    virtual ~NeoResultChunk();



  private:
    /**
      * Appends the newXML to the existingXML. The size of existingXML is increased to hold the
      * newXML. If existingXML == NULL it will allocate the space required to hold the newXML. 
	  * 
	  * @param existingXML the exisiting string that we are appending to, it will be enlarge
	  * to accomodate the newXML.
	  * @param newXML the XML to append
      */
  void appendXML(char** existingXML, const char* const newXML);



	/**
      * Deletes this NeoResultChunk's Axis Elements if loaded. The metadata for
      * the NeoResultChunk is preserved. Returns true if the AxisElements were destroyed,
	  * false if the NeoResultChunk does not contain AxisElements (either never loaded or
	  * loaded and previously destroyed). 
	  */
    const bool destroyAxisElements();


    // Which NeoResultChunk is this in the overall sequence of chunks for this
    // NeoQueryResult. 
    int m_ID;





    // Which NeoQueryResult owns this NeoResultChunk, need to identify the proper
	// NeoResultChunk in the case where the user has multiple NeoQueryResults 
	// active.
	int m_neoQueryResultID;



    // Each chunk provides a range of cursors based upon the target axis
    // specified in the query that produced its result.  The startCursor is
    // the lowest cursor position that this chunk contains. 
    int m_elementOffset;


    // Holds the current state of this NeoResultChunk
    NeoResultChunkState m_state;


    // The axis elements in XML form
    vector<char*>* m_axisElements;



    // How many Axis Elements can this NeoResultChunk hold when fully loaded
	int m_axisElementSize; 



    // When reconstructing XML what is the size of array that is needed to reconstruct
	// the XML. 
	int m_reconstructionSize;



	// Variables needed to communicate with the NeoCoreXMS Server when loading or
	// destroying NeoResultChunk(s).
	UrlConnection* m_urlConnection; 


	// Session ID. 
	char* m_sid; 



	//Did they have a transfer mode of 1, autoload chunks that are in the NEVER_LOADED
	//State selected in the NeoResultProfile. 
	bool m_autoLoadEnabled;

};



#endif /* RESULTCHUNK_H_INCLUDED_C3B72EEB */

