#ifndef NeoResultProfile_H_INCLUDED_C3BA5A28
#define NeoResultProfile_H_INCLUDED_C3BA5A28

#ifdef WIN32
	#ifndef SERVER_CHUNKING
		#include "API_C_dll.h"
		#define	EXPORT_NEORESULTPROFILE
	#else
		#define	HTTPCLIENT4_API
		#define	EXPORT_NEORESULTPROFILE __declspec(dllexport)
	#endif
#else
	#define	HTTPCLIENT4_API
	#define	EXPORT_NEORESULTPROFILE
#endif


#include "NeoException.h"

#ifdef WIN32
#pragma warning( disable : 4290 )
#define UNSIGNED_LONG_FORMAT "%I64u"
#define SIGNED_LONG_FORMAT "%I64d"
#define HEX_LONG_FORMAT "%I64x" 
#else
#define UNSIGNED_LONG_FORMAT "%llu"
#define SIGNED_LONG_FORMAT "%lld"
#define HEX_LONG_FORMAT "%llx"
#endif


#ifndef NXML_RISC
#ifdef WIN32
#define NXML_RISC 0
#else
#define NXML_RISC 1
#endif
#endif

#if NXML_RISC == 1

#ifndef __int64

#ifndef _LP64
#define __int64 long long
#else
#define __int64 long
#endif

#endif

#endif



const char TAG_AEPC[]		     =	"AxisElementsPerChunk";
const char TAG_MRAE[]		     =	"MaxResultAxisElements";
const char TAG_ICP[]			 = 	"InitialCursorPosition";
const char TAG_RCCM[]			 = 	"ResultChunkCaptureMode";
const char TAG_ERM[]			 =	"ExceededResultsMode";
const char TAG_RCTM[]			 = 	"ResultChunkTransferMode";
const char TAG_RCDM[]			 = 	"ResultChunkDiscardMode";


 /** 
   * @memo <DL><DD>Profiles how a NeoQueryResult should be processed by the NeoCore XMS Server and
   * Client.</DD></DL>
   * @doc <br/>Profiles how a NeoQueryResult should be processed by the NeoCore XMS Server and
   * Client.
   *
   * @author Richard A. Monteleone
   *
   * @version 1.0
   *
   * @see NeoConnection#queryXML
   * @see NeoConnection#queryXMLUpdateIntent
   * @see SessionManagedNeoConnection#queryXML
   * @see SessionManagedNeoConnection#queryXMLUpdateIntent
   * @see NeoQueryResult
   * @see NeoResultChunk
   * @see NeoCursor
   */
  
class HTTPCLIENT4_API EXPORT_NEORESULTPROFILE NeoResultProfile
{
  public:


	  enum  captureMode    { SNAPSHOT = 0, CONNNECTED = 1 }; 
	  enum  exceededMode   { REJECT = 0, TRUNCATE = 1 };
	  enum  autoManualMode { MANUAL = 0, AUTO = 1 };


    /** 
      * @memo <DL><DD>Constructor for a NeoResultProfile which is responsible for maintaining 
	  * Chunking/Cursoring configuration.</DD></DL>
      * @doc <br/>Constructor for a NeoResultProfile which is responsible for maintaining 
	  * Chunking/Cursoring configuration.
	  *
	  * @param axisElementsPerChunk  The number of nodes to return in each chunk.<br/>
	  * <UL><LI><B>0</B> = Pack as many axis elements into each chunk as possible, Max-Fit chunking.
      *        In this mode the cursor is always placed at position 0, initialCursorPosition value is
      *        ignored. This is required to prevent us from having to place the first chunk on disk.</LI>
	  *     <LI><B>> 0</B> = The number of elements that the client wants each chunk of the
      *        result to consist of, Uniform chunking.</LI></UL>
	  *
      * @param maxResultAxisElements  The maximum number of nodes to return. <br/>
	  *  <UL><LI><B>0</B> = No maximum, all results are collected and chunked.</LI>
	  *      <LI><B>&gt; 0</B> = The maximum allowable result set size in target axis elements.
      *		   This setting pertains to the entire result. If set then exceededResultsMode goes into
      *        effect.</LI></UL>
      *
	  * @param initialCursorPosition  Determines where the NeoCursor will be placed initially for
	  * the NeoQueryResult. As you might expect this also determines which NeoResultChunk is
      * initally transferred to the client.  
      * If the NeoCursor position does not exist in the result the NeoCursor will be 
	  * placed at position 0 (beginning) by default. <br/>
	  *
	  * @param resultChunkCaptureMode  Governs how results are captured and processed in the cases 
	  * where the results are chuncked/buffered on the server. 
      * 
      *   <UL><LI><B>0 = SNAPSHOT</B> - The results are obtained from the server as a static snapshot.  
	  *                 The result chunks reside on the servers disk until they are deleted. 
	  *                 Once the snapshot has been taken any subsequent changes made will 
	  *                 not be reflected in the snapshot</LI>
      *
      *   <LI><B>1 = CONNECTED</B> - Not currently implemented. </LI></UL>
	  *
	  * @param exceededResultsMode  In the case where the resultSize is larger than the maxResultAxisElements,
      * the client can elect to determine how the result should be processed as follows:
      *
      *   <UL><LI><B>0 = REJECT</B> -  The result is not transferred to the client, the
      *       exception NeoQueryResultExceededMaxException is thrown.</LI>
      *   <LI><B>1 = TRUNCATE</B> The resuilt is truncated to the number of nodes
      *       specified in maxResultSize parameter.</LI></UL>
	  *
      * @param resultChunkTransferMode  Specifies what Client-Server communication
	  * takes place when a NeoCursor has reached the boundary point (the high or low node in the currently loaded chunk), 
	  * or an attempt is made to get XML on a NeoResultChunk that is not loaded. <br/>
      * 
      *   <UL><LI><B>0 = MANUAL</B> - Informs the user that they have reached the boundary
      *       point of the Result.   The next NeoResultChunk must be loaded manually by calling
	  *       loadNeoResultChunk() before one can enter the next NeoResultChunk.</LI>
      *   <LI><B>1 = AUTO</B> - When the boundary (the high or low node in the currently loaded chunk) of a 
	  *       NeoResultChunk is reached NeoQueryResult automatically loads the next NeoResultChunk without notifying the
      *       Client.</LI></UL>
	  *
	  * @param resultChunkDiscardMode  Controls when to release Client resources that are
	  *  being used to hold NeoResultChunks. </br>
      * 
      *   <UL><LI><B>0 = MANUAL</B> - Client assumes the responsibility to manually delete NeoResultChunks by calling destroyNeoResultChunk().</LI> 
      *   <LI><B>1 = AUTO</B> - The client NeoResultChunk is automatically deleted when the 
	  *       NeoCursor exits the scope of a NeoResultChunk.</LI></UL>
	  *
	  * @exception NeoInvalidArgumentException - If any of the parameters are invalid.
      *
	  */
  NeoResultProfile(const int axisElementsPerChunk, const __int64 maxResultAxisElements = 0, 
                   const __int64 initialCursorPosition = 0, const short resultChunkCaptureMode = SNAPSHOT, 
                   const short exceededResultsMode = TRUNCATE, const short resultChunkTransferMode = AUTO,
				   const short resultChunkDiscardMode = AUTO)
  throw (NeoInvalidArgumentException);




     /**
      * @memo <DL><DD>Constructs a NeoResultProfile from the XML provided.  </DD></DL>
      * @doc <br/>Constructs a NeoResultProfile from the XML provided.</br>
	  * 
	  * @param neoResultProfileXML The XML document is as follows:<br/><br/>
	  *  
  	  *	  &lt;NeoResultProfile&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	  	   &lt;AxisElementsPerChunk&gt;0&lt;/AxisElementsPerChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;MaxResultAxisElements&gt;100000&lt;/MaxResultAxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;InitialCursorPosition&gt;0&lt;/InitialCursorPosition&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ResultChunkCaptureMode&gt;0&lt;/ResultChunkCaptureMode&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ExceededResultsMode&gt;1&lt;/ExceededResultsMode&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ResultChunkTransferMode&gt;1&lt;/ResultChunkTransferMode&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ResultChunkDiscardMode&gt;1&lt;/ResultChunkDiscardMode&gt;<br/>
	  *   &lt;/NeoResultProfile&gt;<br/>
	  *  
	  * @exception NeoInvalidArgumentException - If the neoResultProfileXML parameter is not in accordance with the 
	  *              NeoResultProfile XML above. 
	  */
    NeoResultProfile(const char* neoResultProfileXML)
    throw (NeoInvalidArgumentException);


   /**
    * @memo <DL><DD>Sets the axisElementsPerChunk for this object.</DD></DL>
    * @doc <br/>Sets the axisElementsPerChunk for this object.
	* 
	* @param value  The number of axis elements (nodes) to return in each result chunk.
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values.
	*
	* @exception NeoInvalidArgumentException - If the value is invalid.
    */
	void set_axisElementsPerChunk(int value)
    throw (NeoInvalidArgumentException);
	
   /**
    * @memo <DL><DD>Sets the resultChunkTransferMode for this object.</DD></DL>
	* @doc <br/>Sets the resultChunkTransferMode for this object.
	* 
	* @param value  Specifies what Client-Server communication
	* takes place when a NeoCursor has reached the boundary point (the high or low node in the currently loaded chunk), 
	* or an attempt is made to get XML on a NeoResultChunk that is not loaded. <br/>
	* 
	*   <UL><LI><B>0 = MANUAL</B> - Informs the user that they have reached the boundary
	*       point of the Result.   The next NeoResultChunk must be loaded manually by calling
	*       loadNeoResultChunk() before one can enter the next NeoResultChunk.</LI>
	*   <LI><B>1 = AUTO</B> - When the boundary (the high or low node in the currently loaded chunk) of a 
	*       NeoResultChunk is reached NeoQueryResult automatically loads the next NeoResultChunk without notifying the
	*       Client.</LI></UL>
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values.
	*
	* @exception NeoInvalidArgumentException - If value is invalid.
    */
    void set_resultChunkTransferMode(short value)
    throw (NeoInvalidArgumentException);

   /**
    * @memo <DL><DD>Sets the exceededResultsMode for this object.</DD></DL>
    * @doc <br/>Sets the exceededResultsMode for this object.
	*
	* @param value  In the case where the resultSize is larger than the maxResultAxisElements,
    * the client can elect to determine how the result should be processed as follows:
    *
    *   <UL><LI><B>0 = REJECT</B> -  The result is not transferred to the client, the
    *       exception NeoQueryResultExceededMaxException is thrown.</LI>
    *   <LI><B>1 = TRUNCATE</B> The resuilt is truncated to the number of nodes
    *       specified in maxResultSize parameter.</LI></UL>
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values.
	*
	* @exception NeoInvalidArgumentException - If value is invalid.
    */
    void set_exceededResultsMode(short value)
    throw (NeoInvalidArgumentException);

   
   /**
    * @memo <DL><DD>Sets the initialCursorPosition for this object.</DD></DL>
    * @doc <br/>Sets the initialCursorPosition for this object.
	*
	* @param value   Determines where the NeoCursor will be placed initially for
	* the NeoQueryResult. As you might expect this also determines which NeoResultChunk is
    * initally transferred to the client.  
    * If the NeoCursor position does not exist in the result the NeoCursor will be 
	* placed at position 0 (beginning) by default. <br/>
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values
	*
	* @exception NeoInvalidArgumentException - If value is invalid
    */
    void set_initialCursorPosition(__int64 value)
    throw (NeoInvalidArgumentException);


   /**
    * @memo <DL><DD>Sets the maxResultAxisElements for this object.</DD></DL>
    * @doc <br/>Sets the maxResultAxisElements for this object.
	*
	* @param value  The maximum number of nodes to return. <br/>
	*  <UL><LI><B>0</B> = No maximum, all results are collected and chunked.</LI>
	*      <LI><B>&gt; 0</B> = The maximum allowable result set size in target axis elements.
    *		   This setting pertains to the entire result. If set then exceededResultsMode goes into
    *        effect.</LI></UL>
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values
	*
	* @exception NeoInvalidArgumentException - If value is invalid
    */
    void set_maxResultAxisElements(__int64 value)
    throw (NeoInvalidArgumentException);

	
   /**
    * @memo <DL><DD>Sets the resultChunkCaptureMode for this object.</DD></DL>
    * @doc <br/>Sets the resultChunkCaptureMode for this object.
	*
	* @param value  Governs how results are captured and processed in the cases 
	  * where the results are chuncked/buffered on the server. 
      * 
      *   <UL><LI><B>0 = SNAPSHOT</B> - The results are obtained from the server as a static snapshot.  
	  *                 The result chunks reside on the servers disk until they are deleted. 
	  *                 Once the snapshot has been taken any subsequent changes made will 
	  *                 not be reflected in the snapshot</LI>
      *
      *   <LI><B>1 = CONNECTED</B> - Not currently implemented. </LI></UL>
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values
	*
	* @exception NeoInvalidArgumentException - If value is invalid
    */
    void set_resultChunkCaptureMode(short value)
    throw (NeoInvalidArgumentException);

   /**
    * @memo <DL><DD>Sets the resultChunkDiscardMode for this object.</DD></DL>
    * @doc <br/>Sets the resultChunkDiscardMode for this object.
	*
	* @param value - Controls when to release Client resources that are
	*  being used to hold NeoResultChunks. </br>
    * 
    *   <UL><LI><B>0 = MANUAL</B> - Client assumes the responsibility to manually delete NeoResultChunks by calling destroyNeoResultChunk().</LI> 
    *   <LI><B>1 = AUTO</B> - The client NeoResultChunk is automatically deleted when the 
	*       NeoCursor exits the scope of a NeoResultChunk.</LI></UL>
	*
	* @see NeoResultProfile#NeoResultProfile documentation for valid values
	*
	* @exception NeoInvalidArgumentException - If value is invalid
    */
    void set_resultChunkDiscardMode(short value)
    throw (NeoInvalidArgumentException);



#ifndef SERVER_CHUNKING
	/**
	  * @memo <DL><DD>This should be called to free each result.</DD></DL>
      * @doc <br/>Since the XML returned from methods on the NeoResultProfile are allocated from the heap it 
	  * should be freed by the client when no longer needed, releaseXML is provided to the client 
	  * to accomplish this task	
	  *
	  * @param XMLBuffer the buffer that you wish to release, this parameter is assigned NULL
	  * after it is released. 
	  *
	  * @return void
	  */
	void releaseXML(char* XMLBuffer);

// Since this method allocates off the heap I don't expose it to the server which also
// uses this class. 
    /**
	  * @memo <DL><DD>Returns the XML representation of this.</DD></DL>
      * @doc <br/>Returns the XML representation of this.  When finished \
	  * with the returned XML (char*) call releaseXML(char*) to free the XML. 	
	  *
      * @return the XML equivalent of this NeoResultProfile, example XML:<br/><br/>
	  *
  	  *	  &lt;NeoResultProfile&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	  	   &lt;AxisElementsPerChunk&gt;1000&lt;/AxisElementsPerChunk&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;MaxResultAxisElements&gt;100000&lt;/MaxResultAxisElements&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;InitialCursorPosition&gt;0&lt;/InitialCursorPosition&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ResultChunkCaptureMode&gt;1&lt;/ResultChunkCaptureMode&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ExceededResultsMode&gt;1&lt;/ExceededResultsMode&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ResultChunkTransferMode&gt;1&lt;/ResultChunkTransferMode&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	  *	       &lt;ResultChunkDiscardMode&gt;1&lt;/ResultChunkDiscardMode&gt;<br/>
	  *   &lt;/NeoResultProfile&gt;
	  *
	  * @see releaseXML 
	  *
	  * @exception NeoClientException - If an unexpected client error occurs.  
	  */
    char* get_XML() const
	throw (NeoClientException);
#endif

   /**
    * @memo <DL><DD>Gets the initialCursorPosition setting.</DD></DL>
    * @doc <br/>Gets the initialCursorPosition setting.
	*
    * @return The initialCursorPosition setting.
    */
    const __int64 get_initialCursorPosition() const;

   /**
    * @memo <DL><DD>Returns the number of axis elements (nodes) per Chunk.</DD></DL>
    * @doc <br/>Returns the number of axis elements (nodes) per Chunk.
	*
    * @return The axisElementsPerChunk setting.
    */
    const int get_axisElementsPerChunk() const;

   /**
    * @memo <DL><DD>Returns the resultChunkTransferMode setting.</DD></DL>
    * @doc <br/>Returns the resultChunkTransferMode setting.
	*
    * @return The resultChunkTransferMode setting.
    */
    const short get_resultChunkTransferMode() const;

   /**
    * @memo <DL><DD>Returns the maximum number of axis elements (nodes).</DD></DL>
    * @doc <br/>Returns the maximum number of axis elements (nodes).
	*
    * @return The maxResultAxisElements setting.
    */
    const __int64 get_maxResultAxisElements() const;

   /**
    * @memo <DL><DD>Returns the resultChunkCaptureMode setting.</DD></DL>
    * @doc <br/>Returns the resultChunkCaptureMode setting.
	*
    * @return The resultChunkCaptureMode setting.
    */
    const short get_resultChunkCaptureMode() const;

   /**
    * @memo <DL><DD>Returns the resultChunkDiscardMode setting.</DD></DL>
    * @doc <br/>Returns the resultChunkDiscardMode setting.
	*
    * @return the resultChunkDiscardMode setting.
    */
    const short get_resultChunkDiscardMode() const;

   /**
    * @memo <DL><DD>Returns the exceededResultsMode setting.</DD></DL>
    * @doc <br/>Returns the exceededResultsMode setting.
	*
    * @return The exceededResultsMode setting.
    */
    const short get_exceededResultsMode() const;


	// Had to overload the copy constructor since I have default parameters in
	// the constructor. 
	NeoResultProfile(const NeoResultProfile& copyNRP);


    virtual ~NeoResultProfile();



  private:


	char*  getTagValue(const char* targetXML, const char* tag, char* valueHolder)
	throw (NeoInvalidArgumentException);

    int m_axisElementsPerChunk;

    short m_resultChunkTransferMode;

    short m_exceededResultsMode;

    __int64 m_initialCursorPosition;
 
    __int64 m_maxResultAxisElements;

    short m_resultChunkCaptureMode;

    short m_resultChunkDiscardMode;
};


#endif /* NeoResultProfile_H_INCLUDED_C3BA5A28 */
