// NeoACAdmin.h: interface for the NeoACAdmin class.
//
//////////////////////////////////////////////////////////////////////

#ifndef NEO_TRANSFORM_H
#define NEO_TRANSFORM_H

#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif

#include "API_C_dll.h"
#include "API_LEM.h"
#include "NeoConnection.h"
#include "SessionManagedNeoConnection.h"
#include "NeoException.h"


/** 
  * @memo <DL><DD>A class used to transform the results of a query.</DD></DL>
  * @doc <br/>This class allows a user to query the XMS (XML Management System) and have the results transformed by an XSLT transform.
  */
class HTTPCLIENT4_API NeoTransform  
{
	friend class NeoTransformConnectionGuard;
public:

    /**
	 * @memo <DL><DD>This constructor creates an instance of the NeoTransform class using a SessionManagedNeoConnection.</DD></DL>
	 * @doc <br/>This constructor creates an instance of the NeoTransform class using a SessionManagedNeoConnection.
	 *
     * @param smnc The pointer to a logged-in instance of a SessionManagedNeoConnection.
     */
	NeoTransform(SessionManagedNeoConnection* smnc)
		throw (NeoInvalidArgumentException,
			   NeoInvalidSessionException);

    /**
	 * @memo <DL><DD>This constructor creates an instance of the NeoTransform class using a NeoConnection and a session ID.</DD></DL>
	 * @doc <br/>This constructor creates an instance of the NeoTransform class using a NeoConnection and a session ID.
	 *
     * @param smnc The pointer to a logged-in instance of a NeoConnection.
     * @param sid The session ID you get from calling login on the NeoConnection.
     */
	NeoTransform(NeoConnection* nc, char* sid)
		throw (NeoInvalidArgumentException,
			   NeoInvalidSessionException);
	
	/**
	 * @memo <DL><DD>Cleans up memory and resources allocated by this object.</DD></DL>
	 * @doc <br/>Cleans up memory and resources allocated by this object.
	 */
	~NeoTransform();


	/**
	 * @memo <DL><DD>This will query the XMS and transform the results.</DD></DL>
	 * @doc <br/>This will query the XMS and transform the results.
     *
     * @param xquery The XQuery statement to be executed by XMS
     * @param xslt An identifier for the XSLT.
     * @return The results of the query transformed by the stylesheet. 
     */
    char* transformXML(const char* xquery, 
					   const char* xslt)
		throw (NeoException,
			   NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException);

private:
	////////////////////
	//Member Variables
	////////////////////

    SessionManagedNeoConnection* m_smNeoConnection;
    NeoConnection* m_neoConnection;
	char m_sid[128];
	bool m_bIsSessionManaged;



	////////////////////
	//Member Functions
	////////////////////

    NeoConnection* getConnection()
        throw (NeoInvalidArgumentException,  
               NeoPoolClosedException, 
               NeoConnectionPoolException);

    void returnConnection(NeoConnection* nc);

	API_LEM* lem;

};

class NeoTransformConnectionGuard
{
public:
	inline NeoTransformConnectionGuard (NeoTransform* transform, bool bIsSessionManaged = false) 
		: m_nc(NULL), m_Transform(transform), entered(false), m_bIsSessionManaged(bIsSessionManaged)
	{
	}

	inline void enter() 
	{
		m_nc = m_Transform->getConnection();
		entered = true;
	}

	inline void exit() 
	{
		m_Transform->returnConnection(m_nc);
		m_nc = NULL;
		entered = false;
	}

	inline NeoConnection* getNC()
	{
		return m_nc;
	}

	inline ~NeoTransformConnectionGuard() 
	{
		if (entered)
			exit();
	}

private:
	bool entered;
	bool m_bIsSessionManaged;
	NeoTransform* m_Transform;
	NeoConnection* m_nc;

	
};

#endif // NEO_TRANSFORM_H
