//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef URL_CONNECTION_H
#define URL_CONNECTION_H

#ifdef WIN32
#include <winsock2.h>
#include "winnt.h"
#else
#define SOCKET_ERROR -1
#define INVALID_SOCKET -1
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/timeb.h>
#endif
#include <stdio.h>
#include <time.h>

#include "API_LEM.h"
#include "NeoAPICriticalSection.h"
#include "BufferedSender.h"

#define FILE_BUFFER_SIZE (32 * 1024)
#define BOUNDARY_BUFFER_SIZE 50
#define SEGMENT_BUFFER_SIZE 512
#define NORMAL_RESULT_SIZE 4096

const int ERROR_MSG_SIZE = 256;



#ifndef WIN32
#define SOCKET int
#endif

class UrlConnection
{
	struct sockaddr_in localAddr;
	struct sockaddr_in serverAddr;
	SOCKET clientSocket;
	struct hostent *hostPointer;
	char serverName[256];
	int serverPort;
	char localHostName[256];
	bool initialized;
	int socketError;
	int Client_Address_Family;
	int Server_Address_Family;
	int Client_Protocol;
	int Server_Protocol;
	int Socket_Type;
	int sendTries;
	int maxSendTries;
	char protocol_m[64] ;
	char server_m[64] ;
	char date_m[64] ;
	char contentType_m[64] ;
	char contentLength_m[64] ;


	
	// this is the last time that a request was made on this connection
	time_t lastUsedTime; 

	// this is used to protected the socket
	API_NeoCritical* lock;

	int referenceCount; // Used to support refenced counting for this object. 




public:
	UrlConnection (const char* host, const int port) ;
	~UrlConnection(void) ;

	char* getServerName (void) { return serverName; }
	int getServerPort (void) { return serverPort; }

	char* getProtocol() { return protocol_m; }
	char* getHttpServer() { return server_m; }
	char* getDate() { return date_m; }
	char* getContentType() { return contentType_m; }
	int getContentLength() { return atoi(contentLength_m); }

	void setLanguage (const char *language) ;
	void setCharset (const char *charset) ;
	void setRootDir (const char *rootdir) ;

	void releaseBuffer (char* buffer);
    char* storeXML(const char* sid,  const char *sourceFile, const char *schemaURI, const char *prefixFile);
    char* storeXMLString(const char* sid, const char *source, const char *schemaURI, const char *prefix);
	char* queryNoParam(const char* sid, const char *command);
    char* queryOneParam(const char* sid, const char* cmdContext, const char *command, const char *input);
    char* queryTwoParams(const char* sid, const char* cmdContext, const char *command, const char *input, const char *data);
	char* adminRequest (const char* sid, const char* command, const char* request = NULL);
	char* adminRequestPost (const char* sid, const char* command, const char* request = NULL);
	void urlEncode(char *result, const char *input);
    char* executePostMultiPartForm(const char* sid, const char* postContext, 
								   const char* command, 
								   const char* input, 
								   const char* data);
	const int incrementReferenceCount() // When an object hands out a reference to this
										// object it should bump the reference count. 
										// Returns the reference count after it is bumped. 
	{ 	
		lock->enter();
			referenceCount++;
		lock->exit();
		return referenceCount;
	}


	// Support reference counting of this object, since we use this object
	// in NeoQueryResults when it communicates with the server we need reference counting.
	// This function will delete the object if its reference count goes to zero.
	static void releaseUrlConnection (UrlConnection* connectionToDelete);
private:

	API_LEM* lem;

	char* readAllNormalResults(void);
	void prepareHttpStorePrefixBoundary(char *boundary, const char *prefix, const char *CRLF);
	void prepareHttpStoreSchemaBoundary(char *boundary, const char* schema, const char *CRLF);
	void prepareHttpStoreXmlBoundary(char *boundary, const char *xml, const char *CRLF);
	void prepareHttpStoreHeaderBoundary(char *boundary, char *nakedBoundary, const char *CRLF);
	void prepareHttpInfileBoundary(char *infileBoundary, char *headerBoundary);
	void prepareHttpStoreTrailer(char *trailer, char *nakedBoundary, const char *CRLF);
	void prepareHttpStoreHeader(const char* sid, const char *boundaryMarker, const char *bodySize, char *header, const char *CRLF);

	// Get/Post Threshold Multi-Part Form support routines
	void prepareMultiPartFormHeader(const char* sid, const char* postContext, const char* command, const char *boundaryMarker, const char *bodySize, char *header);
	void prepareServerInputHeader(char* serverInputHeader,
								 const char *boundaryMarker, const char *CRLF);
	void prepareServerDataHeader(char* serverDataHeader,
								 const char *boundaryMarker, const char *CRLF);

	void resetFilesForRead(FILE *sourceFileFD, FILE *prefixFileFD);
	int sizeFile(const char *filename);
	int readFile(FILE *inputFile, char *result, int fileSize, BufferedSender& bs);
	int readBuffer(FILE *inputFile, char *readBuff, int buffMax);
	int writeBuffer(char *result);
	int eoResponseMessage(char *buffer);
	void UrlConnection::init();
	void UrlConnection::throwNeoCommunicationException(const char* message, bool format = false);
	bool UrlConnection::CheckConnectionTimeOut();
	void UrlConnection::WakeUpConnection();
	void UrlConnection::closeConnection();



} ;

#endif // URL_CONNECTION_H
